// OscView.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "OscView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


CBaseView::CBaseView()
{
  m_pWndParent = NULL;
  m_pWndView   = NULL;
  return;
}

BOOL CBaseView::Create(CWnd* parent,int nResourceID)
{
  m_pWndParent = parent;
  m_pWndView   = m_pWndParent->GetDlgItem(nResourceID);

  m_pWndView->GetClientRect(&m_rect);

  CClientDC DC(m_pWndView);

  m_memDC.CreateCompatibleDC(&DC);
  m_vwnd.CreateCompatibleBitmap(&DC,GetWidth(),GetHeight());
  m_memDC.SelectObject(&m_vwnd);

  m_brush.CreateStockObject(BLACK_BRUSH);

  m_memDC.SelectStockObject(WHITE_PEN);
  m_memDC.SelectObject(&m_brush);

  CFont font;
  font.CreatePointFont(80,LPCTSTR("font"),&DC);
  m_memDC.SelectObject(&font);

  m_memDC.PatBlt(0,0,GetWidth(),GetHeight(),PATCOPY);

  m_memDC.SetBkMode(TRANSPARENT);
  m_memDC.SetTextColor(RGB(0,0,0));

  return TRUE;
}

void CBaseView::Clear(void)
{
  m_memDC.SelectObject(&m_brush);
  m_memDC.PatBlt(0,0,GetWidth(),GetHeight(),PATCOPY);
  m_memDC.MoveTo(0,GetHeight()/2);
 }

CGdiObject* CBaseView::SelectStockObject(int index)
{
  return m_memDC.SelectStockObject(index);
}

CPen* CBaseView::SelectObject(CPen* pen)
{
  return m_memDC.SelectObject(pen);
}

CBrush* CBaseView::SelectObject(CBrush* brush)
{
  return m_memDC.SelectObject(brush);
}

CPoint CBaseView::MoveTo(int x,int y)
{
  return m_memDC.MoveTo(x,y);
}

BOOL CBaseView::LineTo(int x,int y)
{
  return m_memDC.LineTo(x,y);
}

BOOL CBaseView::SetPixelV(int x,int y,COLORREF color)
{
  return m_memDC.SetPixelV(x,y,color);
}

BOOL CBaseView::PolyLine(POINT* points,int num_points)
{
  return m_memDC.Polyline(points,num_points);
}

BOOL CBaseView::PatBlt(int x0,int y0,int x1,int y1,int op)
{
  return m_memDC.PatBlt(x0,y0,x1,y1,op);
}

void CBaseView::FillSolidRect(int x0,int y0,int x1,int y1,COLORREF color)
{
  m_memDC.FillSolidRect(x0,y0,x1,y1,color);
  return;
}

void CBaseView::OnPaint(void)
{
  m_pWndView->Invalidate(FALSE);
  CPaintDC DC(m_pWndView);
  DC.BitBlt(0,0,m_rect.Width(),m_rect.Height(),&m_memDC,0,0,SRCCOPY);
  return;
}

void CBaseView::RePaint()
{
  CClientDC DC(m_pWndView);
  DC.BitBlt(0,0,GetWidth(),GetHeight(),&m_memDC,0,0,SRCCOPY);
  return;
}

BOOL CBaseView::TextOut(int x,int y,CString& str)
{
  return m_memDC.TextOut(x,y,str);
}

int CBaseView::GetWidth()
{
  return m_rect.Width();
}

int CBaseView::GetHeight()
{
  return m_rect.Height();
}

/////////////////////////////////////////////////////////////////////////////
// COscView

COscView::COscView()
{
}

COscView::~COscView()
{
}

BOOL COscView::Create(CWnd* pParentWnd,int nResourceID, USHORT nBits)
{
  BOOL bres;
  m_nBits = nBits;
  bres = CBaseView::Create(pParentWnd,nResourceID);
  bres = m_sclPen.CreatePen(PS_SOLID,0,RGB(0,128,0));
  bres = m_sigPen.CreatePen(PS_SOLID,0,RGB(0,255,0));
  bres = m_sigPen2.CreatePen(PS_SOLID, 0, RGB(128,128,255));
  bres = DrawScale();

  return bres;
} 


void COscView::Copy(USHORT *vect, USHORT *vect2, ULONG size, ULONG size2)
{
  int height = GetHeight();
  size = min((ULONG)GetWidth(), size);

  UINT maxVal = (1 << m_nBits) - 1;
  double scale = (double) maxVal / (double)height;
  

  m_points.resize(size);
  m_points2.resize(size2);
  for (ULONG i = 0; i < size; i++){
	  m_points[i].x = i;
	  m_points[i].y = (ULONG) ((double)vect[i] / scale);		
  }
  for (i = 0; i < size2; i++){
	  m_points2[i].x = i;
	  m_points2[i].y = (ULONG) ((double)vect2[i] /scale);
  }
}

void COscView::Draw(USHORT *vect, USHORT *vect2, ULONG size, ULONG size2)
{

  DrawScale();

  Copy(vect, vect2, size, size2);

  CPen* old_pen = SelectObject(&m_sigPen);
	if (m_points.size())
		PolyLine(&(m_points[0]),m_points.size());//-m_pos_x);
  SelectObject(&m_sigPen2);
  if (m_points2.size())
		PolyLine(&(m_points2[0]),m_points2.size());
  SelectObject(old_pen);

  RePaint();

  return;
} 

BOOL COscView::DrawScale(void)
{
  int width = GetWidth();// - m_pos_x;
 
  MoveTo(15,GetHeight() / 2);
  LineTo(20,GetHeight() / 2);

  MoveTo(15,GetHeight() / 4);
  LineTo(20,GetHeight() / 4);

  MoveTo(15,GetHeight() * 3 / 4);
  LineTo(20,GetHeight() * 3 / 4);

  CPen* old_pen = SelectObject(&m_sclPen);

  MoveTo(0,GetHeight() / 2);
  LineTo(GetWidth(),GetHeight() / 2);

  MoveTo(0,GetHeight() / 4);
  LineTo(GetWidth(),GetHeight() / 4);

  MoveTo(0,GetHeight() * 3 / 4);
  LineTo(GetWidth(),GetHeight() * 3 / 4);

  MoveTo((width / 2),0);
  LineTo((width / 2),GetHeight());

  MoveTo((width / 4),0);
  LineTo((width / 4),GetHeight());

  MoveTo((width * 3 / 4),0);
  LineTo((width * 3 / 4),GetHeight());

  SelectObject(old_pen);

  return TRUE;
}


BEGIN_MESSAGE_MAP(COscView, CBaseView)
	//{{AFX_MSG_MAP(COscView)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COscView message handlers
