// BoxDOTestPlugin.cpp : Implementation of CBoxDOTestPlugin
#include "stdafx.h"

#include <comdef.h>
#include <string>

#include "Plugin.h"
#include "BoxDOTestPlugin.h"



/////////////////////////////////////////////////////////////////////////////
// CBoxDOTestPlugin


STDMETHODIMP CBoxDOTestPlugin::get_Title(BSTR *pVal)
{
	CComBSTR s("Test boxdo plugin");
	*pVal = s.Detach();
	return S_OK;
}



STDMETHODIMP CBoxDOTestPlugin::AttachDevice(BSTR DeviceID)
{
	_bstr_t sID(DeviceID, true);
	std::string sFileName = "c:\\temp\\1\\" + sID + ".txt";	
	m_OutFile.open(sFileName.c_str());
	m_OutFile << "Attach Device" << LPCTSTR(sID) << "\n";
	return S_OK;
}

STDMETHODIMP CBoxDOTestPlugin::DetachDevice(BSTR DeviceID)
{
	if (!m_OutFile.is_open())
		return E_FAIL;
	m_OutFile << "Detach Device\n";
	m_OutFile.close();

	return S_OK;
}

STDMETHODIMP CBoxDOTestPlugin::OnStart(BSTR DeviceID)
{
	if (!m_OutFile.is_open())
		return E_FAIL;
	m_OutFile << "OnStart\n";


	return S_OK;
}

STDMETHODIMP CBoxDOTestPlugin::OnStop(BSTR DeviceID)
{
	if (!m_OutFile.is_open())
		return E_FAIL;
	m_OutFile << "OnStop\n";

	return S_OK;
}

STDMETHODIMP CBoxDOTestPlugin::OnDataReady(
		/*[in]*/ BSTR DeviceID, 
// Config
//		/*[in]*/ double fTriggerLevel,
		/*[in]*/ double fSamplingTime,
		/*[in]*/ BOOL bInterleaving,		
		/*[in, size_is(2)]*/ BOOL* bChannelEnabled,
		/*[in, size_is(2)]*/ double* fZeroLevel,
		/*[in, size_is(2)]*/ double* fVoltsPerCode,
// Data
		/*[in]*/ int nSize0,
		/*[in, size_is(nSize0)]*/ unsigned short* pData0,
		/*[in]*/ int nSize1,
		/*[in, size_is(nSize1)]*/ unsigned short* pData1)
{
	if (!m_OutFile.is_open())
		return E_FAIL;
	m_OutFile << "OnDataReady\n";
	m_OutFile << "SamplingTime = " << fSamplingTime << "\n";
	m_OutFile << "Interleaving = " << (bInterleaving?"true":"false") << "\n";
	int nDataSize[2] = {nSize0, nSize1};
	WORD* pData[2] = {pData0, pData1};
	for (int i=0; i<2; i++)
	{
		m_OutFile << "ChannelEnabled["<< i << "] = "<< (bChannelEnabled[i]?"true":"false") << "\n";
		m_OutFile << "ZeroLevel["<< i << "] = "<< fZeroLevel[i] << "\n";
		m_OutFile << "VoltsPerCode["<< i << "] = "<< fVoltsPerCode[i] << "\n";
		m_OutFile << "DataSize["<< i << "] = "<< nDataSize[i] << "\n";
		m_OutFile << "Data["<< i << "] = \n";
		for (int j=0; j<nDataSize[i]; j++)
			m_OutFile << pData[i][j] << " ";
		m_OutFile << "\n";
	}
	return S_OK;
}

	struct {ParameterType type; LPCTSTR title; LPCTSTR description;} ParamTypeToStr[] = {
		pvMinimum, "Minimum", " ",
		pvMaximum, "Maximum", " ",
		pvPeakToPeak, "PeakToPeak", "",
		pvAverage, "Average", "",
		pvACRootMeanSquare, "ACRootMeanSquare", " ",
		pvDCRootMeanSquare, "DCRootMeanSquare"," ",
		pvBase, "Base", "",
		pvTop, "Top", "",
		pvAmplitude, "Amplitude", " ()",
		pvFallPreShoot, "FallPreShoot", "  ",
		pvRisePreShoot, "RisePreShoot", "  ",
		pvFallOverShoot, "FallOverShoot", "  ",
		pvRiseOverShoot, "RiseOverShoot", "  ",
		ptPeriod, "Period", "",
		ptFrequency, "Frequency", "",
		ptRiseTime, "RiseTime", " ",
		ptFallTime, "FallTime", " ",
		ptPositiveWidth, "PositiveWidth", ".  (+)",
		ptNegativeWidth,"NegativeWidth", ".  (-)",
		ptDutyCycle, "DutyCycle", "",
		ptMimimumTime, "MimimumTime", "  ",
		ptMaximumTime, "MaximumTime", "  ",
		pmNotCalculated, "NotCalculated", " ",
	};

	struct {ParameterArea area; LPCTSTR title;} ParamAreaToStr[] = {
		paMemory, "Memory",
		paCursors, "Cursors",
		paPeriod, "Period"
	};

STDMETHODIMP CBoxDOTestPlugin::OnParamsReady(/*[in]*/ BSTR DeviceID,
		/*[in]*/ ParameterArea nArea, //   
		/*[in, size_is(2)]*/ double* fCursorTimes, //  
		/*[in]*/ int nCount, //   
		/*[in, size_is(nCount)]*/ ParameterType* pTypes, //  
		/*[in, size_is(nCount)]*/ double* pValues0,
		/*[in, size_is(nCount)]*/ double* pValues1) // 
{
	if (!m_OutFile.is_open())
		return E_FAIL;
	m_OutFile << "Area = " << ParamAreaToStr[nArea].title << "\n";
	m_OutFile << "Cursor1 = " << fCursorTimes[0] << "; Cursors = " << fCursorTimes[1] << "\n";
	m_OutFile << "Parameters Count = " << nCount << "\n";
	m_OutFile << "Parameter\tChannel0\tChannel1\n";
	for (int i=0; i<nCount; i++)
		m_OutFile << ParamTypeToStr[pTypes[i]].description << "\t" << pValues0[i] << "\t" << pValues1[i] << "\n";
	
	return S_OK;
}

