//-----------------------------------------------//
// Unipro oscilloscope driver interface.         //
// Copyrigh (c) BSU 2000-2001                    //
//-----------------------------------------------//

#ifndef __IODOINTF_H__
#define __IODOINTF_H__

#include <windows.h>
#include "..\..\common\include\devtypes.h" // common unipro fuctions and structures

#define	PALSECAMNumberStrings		625
// TV trigger, pal,secam modes strings count

#define	NTSCNumberStrings			525
// TV trigger, NTSC modes strings count

#define	BoxDOChannelsCount			2
// Unipro oscilloscope channels count

#define	BoxDOADCCodeMax				0xFF
// Maximal ADC code

#define BoxDOZeroLevelCodeMax		0xfff
#define BoxDOZeroLevelCodeMiddle	(BoxDOZeroLevelCodeMax / 2)
// Zero level codes range

#define	BoxDOClockFrequency			100e6
// Unipro oscilloscope clock freqency

#define	BoxDODeviceDataSize			0x10000
// Maximal samples count value (per channel)

#define	BoxDOFrequencyMin			(-6)
// Minimal frequency index value 

#define	BoxDOFrequencyMax			21
// Maximal frequency index value

#define	BoxDOSegmentSizeCodeMin		9
// Minimal segment size index value

#define	BoxDoSegmentSizeCodeMax		16
// Maximal segment size index value

#define	BoxDOTriggerLevelCodeMax	0x0FFF
#define	BoxDOTriggerLevelCodeMiddle	(BoxDOTriggerLevelCodeMax / 2)

// Maximal trigger level code value

#define	BoxDOVoltsMax				10
// Maximal gain value (minimal is 0)


#ifdef __cplusplus
extern "C" {
#endif

typedef BYTE CalibratorType; //  

const CalibratorType ctOff = 0;
const CalibratorType ctChannelGroundOutputDAC = 1;
const CalibratorType ctChannel1_10Output1kHz = 2;
const CalibratorType ctChannel1_5Output1kHz = 3;
const CalibratorType ctChannelGroundOutput1kHz = 4;


typedef BYTE ChannelState;

const ChannelState csDisabled = 0;
const ChannelState csEnabled = 1;


typedef BYTE TimeTriggerMode;

const TimeTriggerMode ttmPositiveSlopePeriod = 0;
const TimeTriggerMode ttmNegativeSlopePeriod = 1;
const TimeTriggerMode ttmPositivePulseLength = 2;
const TimeTriggerMode ttmNegativePulseLength = 3;


typedef BYTE TriggerCoupling;

const TriggerCoupling tcDC = 0;
const TriggerCoupling tcDCLowFrequency = 1;
const TriggerCoupling tcAC = 2;
const TriggerCoupling tcACLowFrequency = 3;
const TriggerCoupling tcACHighFrequency = 4;

typedef BYTE TriggerSource;

const TriggerSource tsChannel0 = 0;
const TriggerSource tsChannel1 = 1;
const TriggerSource tsExternal = 2;
const TriggerSource tsExternalDivider1_10 = 3;
const TriggerSource tsPower50Hz = 4;


typedef BYTE TriggerType;

const TriggerType ttSimple = 0;
const TriggerType ttComplex = 1;
const TriggerType ttTV = 2;


typedef BYTE TVTriggerMode;

const TVTriggerMode tvmField1 = 0;
const TVTriggerMode tvmField2 = 1;
const TVTriggerMode tvmFields = 2;
const TVTriggerMode tvmStrings = 3;
const TVTriggerMode tvmString = 4;


typedef BYTE VerticalCoupling;

const VerticalCoupling vcAC = 0;
const VerticalCoupling vcDC = 1;
const VerticalCoupling vcGround = 2;

typedef struct tagChannelIOControlInfo
{
	WORD				*m_pData;/*IN,OUT*/
	// Channel data. filled by driver. memory handling by client
	//  .   .  / 
	 
	ULONG				m_nGain; /*IN*/
	// Gain correction value. Determined with calibration
	//   .    

	ULONG				m_nGainDoubleFreq;/*IN*/
	// Gain in double freq mode
	//        (1 )

	BOOL				m_bGlitch;/*IN, DEAD*/
	// Turn peak detection mode on/off. Absolete.
	//  

	ULONG				m_nProbe;/*IN, DEAD*/
	// Probe value. Absolete
	//  

	ChannelState		m_ChannelState;/*IN*/
	// if state = off - channel data not updated
	//   (/).  ,     , 
	
	VerticalCoupling	m_VerticalCoupling;/*IN*/
	// input type
	//   (//)

    ULONG				m_nVolts; /*IN*/
    // Gain index:
	//   .    [0;BoxDOVoltsMax]
	//         1-2-5
    // 0-0.02 per cell
    // 1-0.05
    // 2-0.1
    // 3-0.2
    // 4-0.5
    // 5-1.0
    //.....
    // BoxDOVoltsMax - 

	int					m_nMemoryMinADCCode;/*OUT*/
	// the least ADC code durng last read
	//    ,       

    int					m_nMemoryMaxADCCode;/*OUT*/
    // the largest ADC code durng last read
	//    ,       

    int					m_nZeroLevel;/*IN*/
    // Hardware zero level code value
	//    (   )

    double				m_fTrueZeroLevel; /*IN, DEAD*/
	// unused. for previouse version compatibility
    // Zero level double value
	//  
} CChannelIOControlInfo;

typedef struct tagOscilloscopeIOControlInfo
{
	CChannelIOControlInfo	m_Channels[BoxDOChannelsCount];
	//  

	BOOL			m_bAdjustment; /*IN*/
	// Adjustment mode. External output signal and channels input signals are defined by CalibratorType field
	//   .           CalibratorType
	
	BOOL			m_bAdjustmentNotReadMemory; /*IN*/
	// if TRUE - channels data not updated. Used for acquiring calibration data:
	// SearchSignalTriggerFrequency, InterleavingValue members
	//  TRUE,       .     
	//   SearchSignalTriggerFrequency, InterleavingValue.

	BOOL			m_bAdjustmentCalibratorMeander; /*IN*/
	// Turns on/off internal meander generator (1 kHz, 1 volt)
	// /    (1 ,  1 )

	int				m_nCalibrator; /*IN*/
	

	CalibratorType	m_CalibratorType; /*IN*/
	// Calibartor type
	//  

	ULONG			m_nDataSize; /*IN, DEAD*/
	//not used. data size depends on m_nSegmentSizeCode member value.
	// .     m_nSegmentSizeCode

	ULONG			m_nDataSizeNotRIS; /*IN, DEAD*/
	// not used
	//  

	ULONG			m_nFrequency; /*IN*/
	// Sampling frequency index (5-2-1 seuence):
	//  .   [0;BoxDOFrequencyMax];
	//         5-2-1;
	//0 - 100MHz
	//1 - 50MHz
	//2 - 20MHz
	//3 - 10MHz
	//4 - 5MHz
	//.......
	//BoxDOFrequencyMax - last valid value

	BOOL			m_bInterleaving;/*IN*/
	// Turns RIS mode on/off
	// /  RIS

	ULONG			m_nInterleavingValue; /*OUT*/
	// Interleaving value. Updated after read with m_bInterleaving = on;
	//     InterleavingValue.
	//       "Interleaving"

	ULONG    		m_nNumberChannels; /*IN*/
	// Channels number. Must be 2 for Unipro oscilloscopes.
	//  .       2;

	int				m_nPrePostTrigger; /*IN*/
	// Pre-post trigger value. Must be in [-DataSize..DataSize] range.
    //  /-  .     [-DataSize..DataSize]

	BOOL			m_bReady; /*OUT*/
	// Data ready flag. Update with IOCTL_READY command. TRUE value indicates that new data have been read.
	//   .   IOCTL_READY.  TRUE ,
	//    .

	ULONG			m_nSegmentSizeCode;
    // Segment size code = exp2(Segment size value)
	//   .   = exp2(m_nSegmentSizeCode);
	//    [BoxDOSegmentSizeCodeMin, BoxDOSegmentSizeCodeMax]

	ULONG			m_nTimeBase; /*IN, DEAD*/
	// Absolete. for previouse version compatibility
	//  

	/////////////// TRIGGER SETUP
	///////////////  
	BOOL			m_bTimeTriggerGreat; /*IN*/
	//     ,    

	double			m_fTimeTriggerLength; /*IN*/
	//  

	double			m_fTimeTriggerFactor; /*IN*/
	//   

	TimeTriggerMode	m_TimeTriggerMode; /*IN*/
	//   

	TriggerCoupling	m_TriggerCoupling; /*IN*/
    // trigger input
	//   

    int				m_nTriggerLevel; /*IN*/
	// Trigger level code
	//   

	double    		m_fTrueTriggerLevel; /*IN, DEAD*/
	// unused. for previouse versions compatibility
	// Trigger level value
	//  

    BOOL			m_bTriggerNormal; /*IN*/
    // trigger normal (TRUE) / auto (FALSE)
	//  (TRUE)/(FALSE)

	BOOL			m_bTriggerSlopeNegative; /*IN*/
    // trigger by raising edge (TRUE) / falling edge (False)
	//    (TRUE - , FALSE - )

    TriggerSource	m_TriggerSource; /*IN*/
    // trigger source
	//  
	
	//////////////// TV TRIGGER SETUP
    TVTriggerMode	m_TVTriggerMode; /*IN*/
	//   

    ULONG			m_nTVTriggerString;/*IN*/
	//  ,   

	BOOL			m_bTVTriggerNTSC; /*IN*/
	//   NTSC

	TriggerType		m_TriggerType; /*IN*/
	//  

	double			m_fSearchSignalTriggerFrequency; /*OUT*/
	//      

    ULONG			m_nAdjustmentZeroLevelChannelID;/*IN*/
	//  ,      

    BOOL			m_bDoubleFreq;/*IN*/
    // 200Mhz enable
	//     (1  * 200)

    WORD			m_nDoubleFreqOffset;/*IN*/
	//          
	//   

	WORD			m_nChargingRateValue;/*IN*/
	//  
	

	BOOL			m_bSyncFlagBool; /*OUT*/
	// Contains current value of sync flag.
	//   
} COscilloscopeIOControlInfo;

#ifdef __cplusplus
}
#endif



/*
Work order:
  IOCTL_GETHARDWAREID
  IOCTL_PLDPROGRAMMING

  loop:
    IOCTL_RUN
    
      IOCTL_READY
    IOCTL_READ

  IOCTL_SHUTDOWN
*/

#define FILE_DEVICE_KRNLDRVR 0x86660000

  // English
  //
  // Common calls
  // Parameters == NULL, if other comments is absent
  //
  // Russian 
  //
  //  
  // Parameters == NULL,    #define	IOCTL_GETHARDWAREID	(FILE_DEVICE_KRNLDRVR | (0 << 2);
#define	IOCTL_SHUTDOWN			(FILE_DEVICE_KRNLDRVR | (2 << 2))

#define	IOCTL_PLDPROGRAMMING	(FILE_DEVICE_KRNLDRVR | (3 << 2))
  // English
  //
  // Parameters - PChar pointer to string:
  //
  // Russian 
  //
  // Parameters -   PChar  :
  
  // English
  //
  // Oscilloscope specific calls
  // Parameters - COscilloscopeIOControlInfo*:
  //
  // Russian 
  //
  //   .
  // Parameters - COscilloscopeIOControlInfo*;
#define	IOCTL_READY						(FILE_DEVICE_KRNLDRVR | (0x101 << 2))
#define	IOCTL_RESET						(FILE_DEVICE_KRNLDRVR | (0x102 << 2))
#define	IOCTL_RUN						(FILE_DEVICE_KRNLDRVR | (0x103 << 2))
#define	IOCTL_READ						(FILE_DEVICE_KRNLDRVR | (0x104 << 2))
#define	IOCTL_ADJUSTMENTTRIGGERLEVEL	(FILE_DEVICE_KRNLDRVR | (0x107 << 2))
#define	IOCTL_SYNCFLAG					(FILE_DEVICE_KRNLDRVR | (0x108 << 2))
#define	IOCTL_UPDATEINTERLEAVING		(FILE_DEVICE_KRNLDRVR | (0x109 << 2))


extern double __stdcall	BoxDOIOFrequencyValue(int nTimeBase); 
//      

extern double __stdcall	BoxDOVoltsValue(int nVolts); 
//         (/)
// 1  - 100 ,  8 

extern BOOL __stdcall	BoxDOIOControl(const CDeviceID& DeviceID, DWORD nIOControlCode, void* pParam);
//  .
// DeviceID -   (devtypes.h)
// nIOControlCode -  
// pParam -   

#endif// __IODOINTF_H__