function [ ] = B132sample( )
clc; clear;
%% definitions
ID_IDX=0; ID_SER=bitshift(1,16); ID_SLOT=bitshift(1,17);
cmAFG=0; cmDDS=1; cmSWEEP=2; cmAM=3; cmFM=4;

%% DLL Loading
if libisloaded('b132drv') unloadlibrary('b132drv'); end; !update.bat;
if ~libisloaded('b132drv') loadlibrary('b132drv','b132drv.h '); end;
if ~libisloaded('b132drv') disp('couldn''t load b132drv.dll'); return; end;
if ( calllib('b132drv','b132_test') ~= 132 ) disp('b132drv.dll test failed'); unloadlibrary 'b132drv'; return; end;

%% Functions and parameters list
libfunctions('b132drv', '-full')

%% Find available B-132 generator devices
count = calllib( 'b132drv', 'b132_device_count' );
if count <= 0 unloadlibrary 'b132drv'; return; end;
disp( sprintf( 'Found %d devices:', count ) );

%% Get found devices information
ser = 0; slot = 0;
for i = 1 : count
    [ err, slot, ser ] = calllib( 'b132drv', 'b132_device_info', i, slot, ser );
    if err == 0 && ser > 0, disp( sprintf( '  Slot%d: B132.%d', slot, ser ) ); end
end

%% Lock first available (optionally you could define ser or slot)
slot = calllib( 'b132drv', 'b132_device_lock', 1 ); % (ID_IDX + index) or (ID_SLOT + slot) or (ID_SER + ser)
if slot < 0 disp('Couldn''t lock B132 device'); unloadlibrary 'b132drv'; return; end;
disp( sprintf( 'Slot%d device is locked', slot ) );

%% Init AFG mode
err = calllib( 'b132drv', 'b132_device_set_mode', slot, cmAFG );
mode = calllib( 'b132drv', 'b132_device_get_mode', slot );
if mode ~= cmAFG disp( sprintf('b132_set_mode() failed (%d)', mode) ); unloadlibrary 'b132drv'; return; end;

Data = zeros( samples, 1, 
sine = libstruct( 'CSineParams' );
sine.amplitude = 1.0; sine.offset = 0.0; sine.period = 10e-6; sine.phase = 0.0;
[err, Data] = calllib( 'b132drv', 'b132_fill_sine_data', sine, ppData, samples );
clear( sine );

%% Finally: unlock device and unload dll
err = calllib( 'b132drv', 'b132_device_unlock', slot );
disp( sprintf( 'Slot%d device is unlocked', slot ) );
unloadlibrary( 'b132drv' );
