#include <stdio.h>
#include <math.h>
#include <windows.h>
#include "..\_include\b132drv.h"
#pragma comment(lib, "..\\_lib\\b132drv.lib")

#define V				///< units visualization: voltage (dummy)
#define mV		* 1e-3	///< units visualization: voltage multiplier constant
#define s				///< units visualization: time (dummy)
#define ms		* 1e-3	///< units visualization: time multiplier constant
#define us		* 1e-6	///< units visualization: time multiplier constant
#define ns		* 1e-9	///< units visualization: time multiplier constant
#define Hz				///< units visualization: frequency (dummy)
#define kHz		* 1e+3	///< units visualization: frequency multiplier constant
#define MHz		* 1e+6	///< units visualization: frequency multiplier constant
#define x03_07	* 2.50	///< pulse signal rise/fall time correction for measuring levels 30%-70%
#define x01_09	* 1.25	///< pulse signal rise/fall time correction for measuring levels 10%-90%

#define ON_EXIT(retcode) {\
	b132_device_unlock( slot );\
	printf( "\n\n           press <ENTER> to exit" );\
	getchar( );\
	return retcode;\
}
#define PAUSE { printf( "\n           press <ENTER> to continue" ); getchar( ); }

//////////////////////////////////////////////////////////////////////////
int test_cmAFG( int slot );
int test_cmDDS( int slot );
int test_cmSWEEP( int slot );
int test_cmAM( int slot );
int test_cmFM( int slot );
int test_cmPM( int slot );

int main(int argc, char* argv[])
{
	int err = NO_ERRORS, count, slot = -1, ser, mode = -1, i = 0;
	printf( "\n=========================================================" );
	printf( "\nUNIPRO GENERATOR B-132 DRIVER (.DLL) TEST, (C)AURIS, 2008" );
	printf( "\n=========================================================" );

	printf( "\n\nB-132 generators found ............................. " );
	count = b132_device_count( );
	if( count <= 0 ) { printf( "0\n" ); ON_EXIT(-1); } else printf( "%d", count );
	while (1) {
		if( b132_device_info( ++i, &slot, &ser ) < 0 ) break;
		printf( "\n  %d: B132.%d in Slot%d", i, ser, slot );
	}
	printf( "\nLock generator ..................................... " );
	slot = b132_device_lock( 0 ); // see {ID_IDX,ID_SER,ID_SLOT} description in b132drv.h
	if( slot < 0 ) { printf( "fail" ); ON_EXIT(-2); } else printf( "done" );

	if( err == NO_ERRORS ) {
		printf( "\n_________________________________________________________" );
		printf( "\nYou could watch A & B output signals with an oscilloscope" );
		printf( "\n2 us/div, 500 mV/div, synchronize it with trigger output" );
		printf( "\n_________________________________________________________" );
		err = test_cmAFG( slot );
	}
	if( err == NO_ERRORS ) {
		printf( "\n_________________________________________________________" );
		err = test_cmDDS( slot );
	}
	if( err == NO_ERRORS ) {
		printf( "\n_________________________________________________________" );
		printf( "\nSet oscilloscope view to 50 us/div, 500 mV/div" );
		printf( "\n_________________________________________________________" );
		err = test_cmSWEEP( slot );
	}
	if( err == NO_ERRORS ) {
		printf( "\n_________________________________________________________" );
		err = test_cmAM( slot );
	}
	if( err == NO_ERRORS ) {
		printf( "\n_________________________________________________________" );
		err = test_cmFM( slot );
	}
	if( err == NO_ERRORS ) {
		printf( "\n_________________________________________________________" );
		err = test_cmPM( slot );
	}
	printf( "\n_________________________________________________________" );
	printf( "\nUnlock generator ................................... " );
	printf( (b132_device_unlock( slot ) == NO_ERRORS) ? "done" : "fail" );
	printf( "\nTest complete." );
	ON_EXIT(err);
}
//////////////////////////////////////////////////////////////////////////
/// Sine signal for cmAFG, cmDDS, cmSWEEP; carrier signal for cmAM, cmFM, cmPM
CSineParams sine = {
	+1.00 V,		// amplitude
	 0.00 V,		// offset
	10.00 us,		// period 10 us
	-pi/2			// -90 degrees (-cos)
};
/// Pulse signal for cmAFG, cmDDS, cmSWEEP
CPulseParams pulse = {
	+2.00 V,		// amplitude
	-1.00 V,		// offset
	10.00 us,		// period 10 us
	-0.02 x01_09,	// leading edge, rise time 200 ns (relative to period duration)
	-0.50,			// {-1..0}: duty cycle 50%, relative to period duration (length 5 us)
	-0.01 x01_09	// trailing edge, fall time 100 ns (relative to period duration)
};
/// Modulating signal for cmAM / AM submode
CTriangleParams triangle_am = {
	+0.50,			// amplitude modulation coef
	+0.25,			// offset level (must be one half of modulation coef)
	 0.10 ms,		// period 100 us
	-0.50			// {-1..0}: relative to period duration, peak position 50 us
};
/// Modulating signal for cmAM / BM submode
CTriangleParams triangle_bm = {
	+2.00,			// balance modulation factor amplitude
	-1.00,			// balance modulation factor offset level
	 0.10 ms,		// period 100 us
	-0.50			// {-1..0}: relative to period duration, peak position 50 us
};
/// Modulating signal for cmFM / FM1 submode
CTriangleParams triangle_fm = {
	 0.60 MHz,		// frequency modulation amplitude
	 0.00 MHz,		// frequency modulation offset level
	 0.10 ms,		// period 100 us
	 -1.00			// {-1..0}: relative to period duration, peak position 100 us
};
/// Modulating signal for cmPM / PM0 submode
CPulseParams pulse_pm = {
	180,			// phase modulation amplitude
	0,				// phase modulation offset level
	0.20 ms,		// period 100 us
	0,				// leading edge
	-0.50,			// {-1..0}: duty cycle 50%
	0				// trailing edge
};

int SignalDataInit(int mode, double signal_length, float **ppData, int *pSamples, double *pTimeBase )
// int mode					{ cmAFG,cmDDS,cmSWEEP,cmAM,cmFM }
// double signal_length		period/duration, s
// float **pData			pointer to first signal data sample pointer
// int *pSamples			required signal data samples count for given signal duration
// int *pTimeBase			sample hold time (sample rate reciprocal)
{
	if( ppData == NULL ) return ERR_PARAM;
	int err;
	if( err = b132_device_get_timing( mode, signal_length, pSamples, pTimeBase ) != NO_ERRORS ) return err;
	if( *ppData != NULL )
		delete [] *ppData;
	*ppData = new float[*pSamples];
	return NO_ERRORS;
}
int SignalDataDeinit( float **ppData )
{
	if( ppData == NULL ) return ERR_PARAM;
	if( *ppData != NULL ) {
		delete [] *ppData;
		*ppData = NULL;
	}
	return NO_ERRORS;
}

//////////////////////////////////////////////////////////////////////////
// B-132 UNIPRO Arbitrary Function Generator example
/// B132drv test in standard arbitrary-function-generator mode
int test_cmAFG( int slot )
{
	int err;
	printf( "\nSet generator to AFG mode .......................... ");
	err = b132_device_set_mode( slot, cmAFG );
	if( b132_device_get_mode( slot ) != cmAFG ) { printf( "fail" ); return ERR_SETMODE; }
	printf( "done" );

	printf( "\nInitialize generator channels ...................... " );
	if( err = b132_chan_set_range( slot, CH_AB, RANGE_2V ) != NO_ERRORS ) { printf( "\nset range error(%d)", err ); return err; }
	if( ( err = b132_chan_set_filter( slot, CH_A, FLT_SIN ) != NO_ERRORS ) ||
		( err = b132_chan_set_filter( slot, CH_B, FLT_OFF ) != NO_ERRORS ) ) { printf( "\nset filter error(%d)", err ); return err; }
	if( err = b132_chan_enable( slot, CH_AB, 1 ) != NO_ERRORS ) { printf( "\nenable channel error(%d)", err ); return err; }
	if( ( err = b132_chan_set_level( slot, CH_A, sine.offset+sine.amplitude*sin(sine.phase) ) != NO_ERRORS )  ||
		( err = b132_chan_set_level( slot, CH_B, pulse.offset ) != NO_ERRORS ) ) { printf( "\ninit level error(%d)", err ); return err; }
	// next overwrites init level for CH_B (init level and pulse modulation uses the same level settings)
//	if( ( err = b132_chan_set_pulse_mod( slot, CH_B, 1, pulse.offset != NO_ERRORS ) ){ printf( "\nset pulse modulation error(%d)", err ); return err; }
	printf( "done" );

	int samples;
	double timebase;
	float *pData; pData = NULL;
	SignalDataInit( cmAFG, sine.period, &pData, &samples, &timebase );
	b132_fill_sine_data( sine, &pData, samples );
	b132_chan_set_data( slot, CH_A, &pData, samples, timebase );
	SignalDataInit( cmAFG, pulse.period, &pData, &samples, &timebase );
	b132_fill_pulse_data( pulse, &pData, samples );
	b132_chan_set_data( slot, CH_B, &pData, samples, timebase );
	SignalDataDeinit( &pData );
	
	if( (err = b132_chan_set_trig( slot, CH_A, 0 us, 1 us, TRIG_USER ) ) != NO_ERRORS ) { printf( "\nset trigger output error(%d)", err ); return err; }

	if(	(err = b132_chan_set_start_delay( slot, CH_AB, 0 us ) ) != NO_ERRORS ) { printf( "\nset start delay error(%d)", err ); return err; }
	
	printf( "\nStart signals generation: A-sine, B-pulse .......... " );
	if( err = b132_chan_start( slot, CH_AB, START_MODE_SOFT_PER ) ) { printf( "fail" ); return err; }
	printf( "done" );

	// Read ADC voltage example (200 averagings in 3 hardware cycles 128+64+8 )
	double volt;
	if( ( err = b132_adc_get_voltage( slot, ADC_CH_3, 200, &volt ) ) != NO_ERRORS )
		printf( "\nget ADC voltage error(%d)", err );

	PAUSE;
	printf( "Stop generation at the end of signals............... " );
	if( err = b132_chan_stop( slot, CH_AB, 1 ) ) { printf( "fail, error %d", err ); return err; }
	int timeout = (int)ceil( max( sine.period, pulse.period ) / (1 ms) ) * 2; // 2 periods
	do {
		Sleep(1);
		if( ( err = b132_chan_get_started( slot, CH_AB ) ) <= 0 ) break;
	} while( timeout-- );
	if( timeout == 0 ) {
		b132_chan_stop( slot, CH_AB, 0 );
		printf("timeout, generation aborted");
	}
	else printf( "done" );

	if( err = b132_chan_enable( slot, CH_AB, 0 ) != NO_ERRORS ) { printf( "\ndisable channel error(%d)", err ); return err; }

	return NO_ERRORS;
}

//////////////////////////////////////////////////////////////////////////
// B-132 UNIPRO Arbitrary Function Generator example ( cmDDS mode )
/// B132drv test in direct digital synthesis mode (fine frequency tune)
int test_cmDDS( int slot )
{
	int err;
	printf( "\nSet generator to DDS mode .......................... ");
	err = b132_device_set_mode( slot, cmDDS );
	if( b132_device_get_mode( slot ) != cmDDS ) { printf( "fail" ); return ERR_SETMODE; }
	printf( "done" );

	printf( "\nInitialize generator channels ...................... " );
	if( ( err = b132_chan_set_range( slot, CH_AB, RANGE_2V ) ) != NO_ERRORS ) { printf( "\nset range error(%d)", err ); return err; }
	if( ( err = b132_chan_set_filter( slot, CH_A, FLT_SIN ) ) != NO_ERRORS ||
		( err = b132_chan_set_filter( slot, CH_B, FLT_OFF ) ) != NO_ERRORS ) { printf( "\nset filter error(%d)", err ); return err; }
	if( ( err = b132_chan_enable( slot, CH_AB, 1 ) ) != NO_ERRORS ) { printf( "\nenable channel error(%d)", err ); return err; }
	if( ( err = b132_chan_set_pulse_mod( slot, CH_A, 0, sine.offset != NO_ERRORS ) ) ||
		( err = b132_chan_set_pulse_mod( slot, CH_B, 0, pulse.offset != NO_ERRORS ) ) ) { printf( "\nset pulse modulation error(%d)", err ); return err; }
	printf( "done" );

	int samples;
	double timebase;
	float *pData; pData = NULL;
	SignalDataInit( cmDDS, sine.period, &pData, &samples, &timebase );
	b132_fill_sine_data( sine, &pData, samples );
	b132_chan_set_data( slot, CH_A, &pData, samples, timebase );
	SignalDataInit( cmDDS, pulse.period, &pData, &samples, &timebase );
	b132_fill_pulse_data( pulse, &pData, samples );
	b132_chan_set_data( slot, CH_B, &pData, samples, timebase );
	SignalDataDeinit( &pData );

	printf( "\nStart signals generation: A-sine, B-pulse .......... " );
	if( ( err = b132_chan_set_trig( slot, CH_A, 0 us, 1 us, TRIG_USER ) ) != NO_ERRORS ) { printf( "\nset trigger output error(%d)", err ); return err; }
	double realfreq;
	if( ( err = b132_chan_set_ddsfreq( slot, CH_A, 1.0/sine.period, &realfreq ) ) != NO_ERRORS ||
		( err = b132_chan_set_ddsfreq( slot, CH_B, 1.0/pulse.period, &realfreq ) ) != NO_ERRORS ) { printf( "\nset DDS frequency error(%d)", err ); return err; }
	if( ( err = b132_chan_start( slot, CH_AB, START_MODE_SOFT_PER ) ) != NO_ERRORS ) { printf( "fail" ); return err; }
	printf( "done" );

	// Read ADC voltage example ( 10 software averaging )
	double volt;
	if( ( err = b132_adc_get_voltage( slot, ADC_CH_3, 10, &volt ) ) != NO_ERRORS )
		printf( "\nget ADC voltage error(%d)", err );

	PAUSE;
	printf( "Change channel A signal frequency +0.1Hz ........... " );
	if( ( err = b132_chan_set_ddsfreq( slot, CH_A, (1.0/sine.period)+0.1, &realfreq ) ) != NO_ERRORS ) { printf( "failed" ); return err; }
	printf( "done" );
	PAUSE;
	printf( "Stop generation (immediate)......................... " );
	if( ( err = b132_chan_stop( slot, CH_AB, 0 ) ) != NO_ERRORS ||
		( err = b132_chan_get_started( slot, CH_AB ) ) != 0 ) { printf( "fail, error %d", err ); return err; }
	printf( "done" );

	if( err = b132_chan_enable( slot, CH_AB, 0 ) != NO_ERRORS ) { printf( "\ndisable channel error(%d)", err ); return err; }

	return NO_ERRORS;
}

//////////////////////////////////////////////////////////////////////////
// B-132 UNIPRO Arbitrary Function Generator example ( cmSWEEP mode )
/// B132drv test in frequency linear sweep mode
int test_cmSWEEP( int slot )
{
	int err;
	printf( "\nSet generator to SWEEP mode ........................ ");
	err = b132_device_set_mode( slot, cmSWEEP );
	if( b132_device_get_mode( slot ) != cmSWEEP ) { printf( "fail" ); return ERR_SETMODE; }
	printf( "done" );
	
	printf( "\nInitialize generator channels ...................... " );
	if( ( err = b132_chan_set_range( slot, CH_AB, RANGE_2V ) ) != NO_ERRORS ) { printf( "\nset range error(%d)", err ); return err; }
	if( ( err = b132_chan_set_filter( slot, CH_A, FLT_SIN ) ) != NO_ERRORS ||
		( err = b132_chan_set_filter( slot, CH_B, FLT_OFF ) ) != NO_ERRORS ) { printf( "\nset filter error(%d)", err ); return err; }
	if( ( err = b132_chan_enable( slot, CH_AB, 1 ) ) != NO_ERRORS ) { printf( "\nenable channel error(%d)", err ); return err; }
	if( ( err = b132_chan_set_pulse_mod( slot, CH_A, 0, sine.offset != NO_ERRORS ) ) ||
		( err = b132_chan_set_pulse_mod( slot, CH_B, 0, pulse.offset != NO_ERRORS ) ) ) { printf( "\nset pulse modulation error(%d)", err ); return err; }
	printf( "done" );

	int samples;
	double timebase;
	float *pData; pData = NULL;
	SignalDataInit( cmSWEEP, sine.period, &pData, &samples, &timebase );
	b132_fill_sine_data( sine, &pData, samples );
	b132_chan_set_data( slot, CH_A, &pData, samples, timebase );
	SignalDataInit( cmSWEEP, pulse.period, &pData, &samples, &timebase );
	b132_fill_pulse_data( pulse, &pData, samples );
	b132_chan_set_data( slot, CH_B, &pData, samples, timebase );
	SignalDataDeinit( &pData );

	printf( "\nStart signals generation: A-sine, B-pulse .......... " );
	if( ( err = b132_chan_set_trig( slot, CH_A, 0 us, 1 us, TRIG_USER ) ) != NO_ERRORS ) { printf( "\nset trigger output error(%d)", err ); return err; }
	int steps = 4, realsteps;
	double realrange, realperiod;
	double freq = 1.0 / sine.period; // base frequency [f]
	double range = 3.0 / sine.period; // sweep range [f..f+3f]
	double sweeptime = 40.0 * sine.period; // 4 frequency points (10 periods each) [f,2f,3f,4f]
	if( ( err = b132_chan_set_sweep( slot, CH_AB, freq, range, sweeptime, steps,
		&realrange, &realperiod, &realsteps ) ) != NO_ERRORS ) { printf( "\nset sweep error(%d)", err ); return err; }
	if( ( err = b132_chan_start( slot, CH_AB, START_MODE_SOFT_PER ) ) != NO_ERRORS ) { printf( "fail" ); return err; }
	printf( "done" );

	// Read ADC voltage example ( 10 software averaging )
	double volt;
	if( ( err = b132_adc_get_voltage( slot, ADC_CH_3, 10, &volt ) ) != NO_ERRORS )
		printf( "\nget ADC voltage error(%d)", err );

	PAUSE;
	printf( "Stop generation (immediate)......................... " );
	if( ( err = b132_chan_stop( slot, CH_AB, 0 ) ) != NO_ERRORS ||
		( err = b132_chan_get_started( slot, CH_AB ) ) != 0 ) { printf( "fail, error %d", err ); return err; }
	printf( "done" );

	if( err = b132_chan_enable( slot, CH_AB, 0 ) != NO_ERRORS ) { printf( "\ndisable channel error(%d)", err ); return err; }

	return NO_ERRORS;
}

//////////////////////////////////////////////////////////////////////////
// B-132 UNIPRO Arbitrary Function Generator example ( cmAM mode )
/// B132drv test in amplitude-modulation mode
int test_cmAM( int slot )
{
	int err;
	printf( "\nSet generator to AM mode ........................... ");
	err = b132_device_set_mode( slot, cmAM );
	if( b132_device_get_mode( slot ) != cmAM ) { printf( "fail" ); return ERR_SETMODE; }
	printf( "done" );

	printf( "\nInitialize generator channels ...................... " );
	if( ( err = b132_chan_set_range( slot, CH_AB, RANGE_2V ) ) != NO_ERRORS ) { printf( "\nset range error(%d)", err ); return err; }
	if( ( err = b132_chan_set_filter( slot, CH_A, FLT_SIN ) ) != NO_ERRORS ||
		( err = b132_chan_set_filter( slot, CH_B, FLT_OFF ) ) != NO_ERRORS ) { printf( "\nset filter error(%d)", err ); return err; }
	if( ( err = b132_chan_enable( slot, CH_AB, 1 ) ) != NO_ERRORS ) { printf( "\nenable channel error(%d)", err ); return err; }
	printf( "done" );

	printf( "\nStart AM (m=50%%) signal generation: ................ " );

	int samples;
	double timebase;
	float *pData; pData = NULL;

	SignalDataInit( cmAM, sine.period, &pData, &samples, &timebase );
	b132_fill_sine_data( sine, &pData, samples );
	err = b132_chan_set_carrier( slot, &pData, samples );
	SignalDataDeinit( &pData );
	if ( err != NO_ERRORS ) { printf( "\nset carrier signal error(%d)", err ); return err; }

	SignalDataInit( cmAM, triangle_am.period, &pData, &samples, &timebase );
	b132_fill_triangle_data( triangle_am, &pData, samples );
	err = b132_chan_set_modulating( slot, AM, &pData, samples );
	SignalDataDeinit( &pData );
	if ( err != NO_ERRORS ) { printf( "\nset modulating signal error(%d)", err ); return err; }

	if( ( err = b132_chan_set_trig( slot, CH_MOD, 0 us, 1 us, TRIG_USER ) ) != NO_ERRORS ) { printf( "\nset trigger output error(%d)", err ); return err; }
	if( ( err = b132_chan_set_ddsfreq( slot, CH_CAR, 1.0 / sine.period, NULL ) ) != NO_ERRORS ||
		( err = b132_chan_set_ddsfreq( slot, CH_MOD, 1.0 / triangle_am.period, NULL ) ) != NO_ERRORS ) { printf( "\nset DDS(AM) frequency error(%d)", err ); return err; }
	if( ( err = b132_chan_start( slot, CH_AB, START_MODE_SOFT_PER ) ) != NO_ERRORS ) { printf( "fail" ); return err; }
	printf( "done" );

	// Read ADC voltage example ( 10 software averaging )
	double volt;
	if( ( err = b132_adc_get_voltage( slot, ADC_CH_3, 10, &volt ) ) != NO_ERRORS )
		printf( "\nget ADC voltage error(%d)", err );

	PAUSE;
	printf( "Start BM signal generation: ........................ " );

	SignalDataInit( cmAM, triangle_bm.period, &pData, &samples, &timebase );
	b132_fill_triangle_data( triangle_bm, &pData, samples );
	err = b132_chan_set_modulating( slot, BM, &pData, samples );
	SignalDataDeinit( &pData );
	if ( err != NO_ERRORS ) { printf( "\nset modulating signal error(%d)", err ); return err; }

	if( ( err = b132_chan_set_ddsfreq( slot, CH_MOD, 1.0 / triangle_bm.period, NULL ) ) != NO_ERRORS ) { printf( "\nset DDS(BM) frequency error(%d)", err ); return err; }
	if( ( err = b132_chan_start( slot, CH_MOD, START_MODE_SOFT_PER ) ) != NO_ERRORS ) { printf( "fail" ); return err; }
	printf( "done" );

	PAUSE;
	printf( "Stop generation (immediate)......................... " );
	
	if( ( err = b132_chan_stop( slot, CH_AB, 0 ) ) != NO_ERRORS ||
		( err = b132_chan_get_started( slot, CH_AB ) ) != 0 ) { printf( "fail, error %d", err ); return err; }
	printf( "done" );

	if( err = b132_chan_enable( slot, CH_AB, 0 ) != NO_ERRORS ) { printf( "\ndisable channel error(%d)", err ); return err; }

	return NO_ERRORS;
}

//////////////////////////////////////////////////////////////////////////
// B-132 UNIPRO Arbitrary Function Generator example ( cmFM mode )
/// B132drv test in frequency-modulation mode
int test_cmFM( int slot )
{
	int err;
	printf( "\nSet generator to FM mode ........................... ");
	err = b132_device_set_mode( slot, cmFM );
	if( b132_device_get_mode( slot ) != cmFM ) { printf( "fail" ); return ERR_SETMODE; }
	printf( "done" );

	printf( "\nInitialize generator channels ...................... " );
	if( ( err = b132_chan_set_range( slot, CH_AB, RANGE_2V ) ) != NO_ERRORS ) { printf( "\nset range error(%d)", err ); return err; }
	if( ( err = b132_chan_set_filter( slot, CH_A, FLT_SIN ) ) != NO_ERRORS ||
		( err = b132_chan_set_filter( slot, CH_B, FLT_OFF ) ) != NO_ERRORS ) { printf( "\nset filter error(%d)", err ); return err; }
	if( ( err = b132_chan_enable( slot, CH_AB, 1 ) ) != NO_ERRORS ) { printf( "\nenable channel error(%d)", err ); return err; }
	printf( "done" );

	printf( "\nStart FM signal generation (100 to 700 kHz sweep): . " );

	int samples;
	double timebase;
	float *pData; pData = NULL;

	SignalDataInit( cmFM, sine.period, &pData, &samples, &timebase );
	b132_fill_sine_data( sine, &pData, samples );
	err = b132_chan_set_carrier( slot, &pData, samples );
	SignalDataDeinit( &pData );
	if ( err != NO_ERRORS ) { printf( "\nset carrier signal error(%d)", err ); return err; }

	SignalDataInit( cmFM, triangle_fm.period, &pData, &samples, &timebase );
	b132_fill_triangle_data( triangle_fm, &pData, samples );
	err = b132_chan_set_modulating( slot, FM1, &pData, samples );
	SignalDataDeinit( &pData );
	if ( err != NO_ERRORS ) { printf( "\nset modulating signal error(%d)", err ); return err; }

	if( ( err = b132_chan_set_trig( slot, CH_MOD, 0 us, 1 us, TRIG_USER ) ) != NO_ERRORS ) { printf( "\nset trigger output error(%d)", err ); return err; }
	if( ( err = b132_chan_set_ddsfreq( slot, CH_CAR, 1.0 / sine.period, NULL ) ) != NO_ERRORS ||
		( err = b132_chan_set_ddsfreq( slot, CH_MOD, 1.0 / triangle_fm.period, NULL ) ) != NO_ERRORS ) { printf( "\nset DDS(FM) frequency error(%d)", err ); return err; }
	if( ( err = b132_chan_start( slot, CH_AB, START_MODE_SOFT_PER ) ) != NO_ERRORS ) { printf( "fail" ); return err; }
	printf( "done" );

	// Read ADC voltage example ( 10 software averaging )
	double volt;
	if( ( err = b132_adc_get_voltage( slot, ADC_CH_3, 10, &volt ) ) != NO_ERRORS )
		printf( "\nget ADC voltage error(%d)", err );

	PAUSE;
	printf( "Stop generation (immediate)......................... " );

	if( ( err = b132_chan_stop( slot, CH_AB, 0 ) ) != NO_ERRORS ||
		( err = b132_chan_get_started( slot, CH_AB ) ) != 0 ) { printf( "fail, error %d", err ); return err; }
	printf( "done" );

	if( err = b132_chan_enable( slot, CH_AB, 0 ) != NO_ERRORS ) { printf( "\ndisable channel error(%d)", err ); return err; }

	return NO_ERRORS;
}

//////////////////////////////////////////////////////////////////////////
// B-132 UNIPRO Arbitrary Function Generator example
/// B132drv test in phase-modulation mode
int test_cmPM( int slot )
{
	int err;
	printf( "\nSet generator to phase modulation mode ............. ");
	err = b132_device_set_mode( slot, cmPM );
	if( b132_device_get_mode( slot ) != cmPM ) { printf( "fail" ); return ERR_SETMODE; }
	printf( "done" );
	
	printf( "\nInitialize generator channels ...................... " );
	if( ( err = b132_chan_set_range( slot, CH_AB, RANGE_2V ) ) != NO_ERRORS ) { printf( "\nset range error(%d)", err ); return err; }
	if( ( err = b132_chan_set_filter( slot, CH_A, FLT_SIN ) ) != NO_ERRORS ||
		( err = b132_chan_set_filter( slot, CH_B, FLT_OFF ) ) != NO_ERRORS ) { printf( "\nset filter error(%d)", err ); return err; }
	if( ( err = b132_chan_enable( slot, CH_AB, 1 ) ) != NO_ERRORS ) { printf( "\nenable channel error(%d)", err ); return err; }
	printf( "done" );
	
	printf( "\nStarting PM signal (sine with phase reversal) ...... " );
	
	int samples;
	double timebase;
	float *pData; pData = NULL;
	
	SignalDataInit( cmPM, sine.period, &pData, &samples, &timebase );
	b132_fill_sine_data( sine, &pData, samples );
	err = b132_chan_set_carrier( slot, &pData, samples );
	SignalDataDeinit( &pData );
	if ( err != NO_ERRORS ) { printf( "\nset carrier signal error(%d)", err ); return err; }
	
	SignalDataInit( cmPM, pulse_pm.period, &pData, &samples, &timebase );
	b132_fill_pulse_data( pulse_pm, &pData, samples );
	err = b132_chan_set_modulating( slot, PM0, &pData, samples );
	SignalDataDeinit( &pData );
	if ( err != NO_ERRORS ) { printf( "\nset modulating signal error(%d)", err ); return err; }
	
	if( ( err = b132_chan_set_trig( slot, CH_MOD, 0 us, 1 us, TRIG_USER ) ) != NO_ERRORS ) { printf( "\nset trigger output error(%d)", err ); return err; }
	if( ( err = b132_chan_set_ddsfreq( slot, CH_CAR, 1.0 / sine.period, NULL ) ) != NO_ERRORS ||
		( err = b132_chan_set_ddsfreq( slot, CH_MOD, 1.0 / pulse_pm.period, NULL ) ) != NO_ERRORS ) { printf( "\nset DDS(PM) frequency error(%d)", err ); return err; }
	if( ( err = b132_chan_start( slot, CH_AB, START_MODE_SOFT_PER ) ) != NO_ERRORS ) { printf( "fail" ); return err; }
	printf( "done" );
	
	// Read ADC voltage example ( 10 software averaging )
	double volt;
	if( ( err = b132_adc_get_voltage( slot, ADC_CH_3, 10, &volt ) ) != NO_ERRORS )
		printf( "\nget ADC voltage error(%d)", err );
	
	PAUSE;
	printf( "Stop generation (immediate)......................... " );
	
	if( ( err = b132_chan_stop( slot, CH_AB, 0 ) ) != NO_ERRORS ||
		( err = b132_chan_get_started( slot, CH_AB ) ) != 0 ) { printf( "fail, error %d", err ); return err; }
	printf( "done" );
	
	if( err = b132_chan_enable( slot, CH_AB, 0 ) != NO_ERRORS ) { printf( "\ndisable channel error(%d)", err ); return err; }
	
	return NO_ERRORS;
}
